#here are the functions for 





"""
Called in the inner loop of `generate_contact_vectors!`, this function removes the degree corresponding to kth entry of index_lists, replaces it with another randomly sampled degree, and returns `csum` adjusted to the new samples. 

We repeat this function until `csum == 0`. 
"""
@inline function reindex!(k,csum,index_list_i,index_list_j,j_to_i_contacts,i_to_j_contacts,sample_list_i,sample_list_j)
    i_index = index_list_i[k]
    j_index = index_list_j[k]    
    csum +=  j_to_i_contacts[j_index] - i_to_j_contacts[i_index] +  sample_list_i[k] -  sample_list_j[k]
    i_to_j_contacts[i_index] = sample_list_i[k] 
    j_to_i_contacts[j_index] = sample_list_j[k]    
    return csum
end


"""
    generate_contact_vectors!(ij_dist<:Distribution,ji_dist<:Distribution,i_to_j_contacts::Vector{T}, j_to_i_contacts::Vector{T})

Fills i_to_j_contacts and j_to_i_contacts with degrees sampled from ij_dist and ji_dist, such that i_to_j_contacts and j_to_i_contacts have equal sum.

Given `μz_i = mean(ij_dist)` and `μ_j = mean(ji_dist)`, these must satisfy `μ_i* length(i_to_j_contacts) == μ_j* length(j_to_i_contacts)`  
"""
function generate_contact_vectors!(ij_dist,ji_dist,i_to_j_contacts::Vector{T}, j_to_i_contacts::Vector{T}) where T
    rand!(RNG,ij_dist,i_to_j_contacts)
    rand!(RNG,ji_dist,j_to_i_contacts)
    l_i = length(i_to_j_contacts)
    l_j = length(j_to_i_contacts)

    csum = sum(i_to_j_contacts) - sum(j_to_i_contacts)

    inner_iter = 10
    index_list_i = @MVector zeros(Int,inner_iter)
    index_list_j = similar(index_list_i)
    sample_list_i = @MVector zeros(T,inner_iter)
    sample_list_j = similar(sample_list_i)
    
    while csum != 0
        sample!(RNG,1:l_i,index_list_i)
        sample!(RNG,1:l_j,index_list_j)
        rand!(RNG,ij_dist,sample_list_i)
        rand!(RNG,ji_dist,sample_list_j)
        @inbounds for i = 1:inner_iter
            if csum != 0
                csum = reindex!(i,csum,index_list_i,index_list_j,j_to_i_contacts,i_to_j_contacts,sample_list_i,sample_list_j)
            end
        end
    end
    return nothing
end