
function get_parameters()
    params = (
        I_0_fraction = 0.01,
        base_transmission_probability = 0.5,
        recovery_rate = 0.1,
        immunization_loss_prob = 0.5,
        π_base = -0.05,
        η = 0.0,
        κ = 0.0,
        ω = 0.0,
        ρ = [0.0,0.0,0.0],
        ω_en = 0.0,
        ρ_en =  [0.0,0.0,0.0],
        γ = 0.0,
        β = 10.0,
        notification_parameter = 0.0,
        vaccinator_prob = 0.2,
        app_user_fraction = 0.5,
    )
    return params
end

function get_u_0(nodes,I_0_fraction,vaccinator_prob)
    #vaccinator with 50% probability
    is_vaccinator = rand(nodes) .> vaccinator_prob
    status = fill(Susceptible,nodes)
    init_indices = rand(RNG, 1:nodes, round(Int,nodes*I_0_fraction))
    status[init_indices] .= Infected #start with five infected
    return status,is_vaccinator
end

struct ModelSolution{T,InfNet,SocNet,WSMixingDist,RestMixingDist}
    sim_length::Int
    nodes::Int
    params::T
    u_next_inf::Vector{AgentStatus}
    u_next_vac::Vector{Bool}
    u_inf::Vector{AgentStatus}
    u_vac::Vector{Bool}
    covid_alert_times::Array{Int,2}
    time_of_last_alert::Vector{Int}
    inf_network::InfNet
    soc_network::SocNet
    index_vectors::Vector{Vector{Int}}
    demographics::Vector{AgentDemographic}
    app_user::Vector{Bool}
    app_user_list::Vector{Int}
    app_user_index::Vector{Int}
    status_totals::Vector{Int}
    status_totals_next::Vector{Int}
    ws_matrix_tuple::WSMixingDist
    rest_matrix_tuple::RestMixingDist
    function ModelSolution(sim_length,params::T,num_households) where T
        demographics,base_network,index_vectors = generate_population(num_households)
        pop_sizes = length.(index_vectors)
        map_symmetrize(m_tuple) = map(md -> symmetrize_means(pop_sizes,md), m_tuple)

        ws_matrix_tuple = map_symmetrize(workschool_mixing)
        rest_matrix_tuple = map_symmetrize(rest_mixing)

        app_user_list = zeros(length(demographics))
        is_app_user = rand(RNG,length(demographics)) .< params.app_user_fraction
        app_user_index = findall(==(true),is_app_user)
        app_user_list[is_app_user] .= collect(1:length(app_user_index))
        nodes = length(demographics)
        
        u_0_inf,u_0_vac = get_u_0(nodes,params.I_0_fraction,params.vaccinator_prob)

        infected_mixing_graph,soc_mixing_graph = time_dep_mixing_graphs(sim_length,base_network,demographics,index_vectors,ws_matrix_tuple,rest_matrix_tuple)
      
        covid_alert_times = zeros(Int,14,length(app_user_index)) #two weeks worth of values
        time_of_last_alert = fill(-1,length(app_user_index)) #time of last alert is negative if no alert has been recieved

        status_totals = [count(==(AgentStatus(i)), u_0_inf) for i in 1:AgentStatus.size]
    
        return new{T,typeof(infected_mixing_graph),typeof(soc_mixing_graph),typeof(ws_matrix_tuple),typeof(rest_matrix_tuple)}(
            sim_length,
            nodes,
            params,
            u_0_inf,
            u_0_vac,
            copy(u_0_inf),
            copy(u_0_vac),
            covid_alert_times,
            time_of_last_alert,
            infected_mixing_graph,
            soc_mixing_graph,
            index_vectors,
            demographics,
            is_app_user,
            app_user_list,
            app_user_index,
            status_totals,
            copy(status_totals),
            ws_matrix_tuple,
            rest_matrix_tuple
        )
    end
end
