
function plot_edges!(plt,g,network_pts, edge_alpha, edge_color)
    xpts = [i[1] for i in network_pts]
    ypts = [i[2] for i in network_pts]
    for edge in edges(g)
        e = [src(edge), dst(edge)]
        plot!(plt,xpts[e],ypts[e], color = edge_color, alpha = edge_alpha)
    end
end

function plot_nodes!(plt,network_pts,u_t)
    for (i,var) in enumerate(instances(AgentStatus))
        indices_of_var = findall(x -> x == var, u_t)
        xpts = [i[1] for i in network_pts]
        ypts = [i[2] for i in network_pts]
        scatter!(plt,xpts[indices_of_var],ypts[indices_of_var]; color = color_palette[i], markersize = 3)
    end
end
function make_layout(g)
    a = LightGraphs.adjacency_matrix(g) 
    # network_pts = Stress.layout(a,Stress.Point2f0) # generate 2D layout
    network_pts = SFDP.layout(a,SFDP.Point2f0,tol=0.1,C=1,K=1,iterations=10)
    return network_pts
end
function plot_model_spatial_gif(static_graph,annealed_graph_list::AbstractVector,solution) 
    plt = plot()
    network_pts = make_layout(static_graph)
    anim = Animation()
        for (t,(u_t,g_t)) in enumerate(zip(solution,annealed_graph_list))
            frame_t = plot()
            plot_edges!(frame_t,g_t,network_pts,0.2,:blue)  ;
            plot_edges!(frame_t,static_graph,network_pts,0.8,:red)

            plot_nodes!(frame_t,network_pts,u_t)
            plot!(frame_t;title = "t = $t", dpi = 300,xticks = nothing,yticks = nothing, legend = false)
            frame(anim,frame_t)
        end        
    gif(anim, joinpath(PACKAGE_FOLDER,"plots", "graphplot.gif"), fps = 1.0)
end
using LabelledArrays
function aggregate_timeseries(sol)
    agent_statuses = map(Symbol,[AgentStatus(i) for i in 1:AgentStatus.size])
    return @LArray [count(x -> AgentStatus(i)  == x, u_t) for u_t in sol,  i in 1:AgentStatus.size] namedtuple(agent_statuses,ntuple(i -> (:,i),length(agent_statuses)))
end

function plot_aggregate_timeseries(sol,network)
    plot(sol,labels = reshape([string(k) for k in keys(keys(sol))],(1,:)))
    
end