# using LoopVectorization
function contact_weight(p, contact_time) 
    return 1 - (1-p)^contact_time
end
Base.@propagate_inbounds @views function update_alert_durations!(t,modelsol)  
    @unpack notification_parameter = modelsol.params
    @unpack time_of_last_alert, app_user_index,inf_network,covid_alert_times,app_user = modelsol
    for (i,node) in enumerate(modelsol.app_user_index), mixing_graph in modelsol.inf_network.graph_list[t]
        for j in 2:14
            covid_alert_times[j-1,i] = covid_alert_times[j,i] #shift them all back  
        end
        for j in neighbors(mixing_graph.g,node)
            
            if app_user[j] 
                covid_alert_times[end,i] += get_weight(mixing_graph,GraphEdge(node,j)) #add the contact times for today to the back
            end
        end
        if rand(RNG) < 1 - (1- notification_parameter)^sum(covid_alert_times[:,i])
            time_of_last_alert[i] = t
        end
    end
end

Base.@propagate_inbounds @views function update_infection_state!(t,modelsol)
    @unpack base_transmission_probability,immunization_loss_prob,recovery_rate = modelsol.params
    @unpack u_inf,u_vac,u_next_inf,u_next_vac,demographics,inf_network,status_totals = modelsol
    
    function agent_transition!(node, from::AgentStatus,to::AgentStatus) 
        status_totals[Int(from)] -= 1
        status_totals[Int(to)] += 1
        u_next_inf[node] = to
    end

    u_next_inf .= u_inf
    for i in 1:modelsol.nodes
        agent_status = u_inf[i]
        is_vaccinator = u_vac[i]
        agent_demo = demographics[i]
        if agent_status == Susceptible
            if is_vaccinator
                agent_transition!(i, Susceptible,Immunized)
            else
                for mixing_graph in inf_network.graph_list[t]
                    for j in neighbors(mixing_graph.g,i)    
                        if u_inf[j] == Infected && u_next_inf[i] != Infected 
                            if rand(RNG) < contact_weight(base_transmission_probability,get_weight(mixing_graph,GraphEdge(i,j)))
                         
                                
                                agent_transition!(i, Susceptible,Infected)
                            end
                        end
                    end
                end
            end
        elseif agent_status == Infected
            if rand(RNG) < recovery_rate
               agent_transition!(i, Infected,Recovered)
            end
        elseif agent_status == Immunized
            if rand(RNG) < immunization_loss_prob
               agent_transition!(i, Immunized,Susceptible)
            end
        end
    end
    # display(u_next_inf)
end
Base.@propagate_inbounds @views function update_vaccination_opinion_state!(t,modelsol,total_infections)
    
    @unpack π_base, η,γ, κ, ω, ρ, ω_en,ρ_en,γ,β = modelsol.params
    @unpack demographics,time_of_last_alert, nodes, soc_network,u_vac,u_next_vac,app_user,app_user_list = modelsol
    app_user_pointer = 0


    for i in 1:nodes
        vac_payoff = 0
        soc_nbrs_vac = @MArray [0,0,0]
        soc_nbrs_nonvac = 0
        num_soc_nbrs = 0
        for sc_g in soc_network.graph_list[t]
            soc_nbrs = neighbors(sc_g.g,i)
            num_soc_nbrs += length(soc_nbrs)
            for nbr in soc_nbrs
                if u_vac[nbr]
                    soc_nbrs_vac[Int(demographics[nbr])] += 1
                else
                    soc_nbrs_nonvac += 1
                end
            end
        end
        vac_payoff += π_base + dot(ρ,soc_nbrs_vac) + total_infections*ω +
            ifelse(num_soc_nbrs> 0, κ * ((sum(soc_nbrs_vac) - soc_nbrs_nonvac/num_soc_nbrs)),0)
        
        if app_user[i] && time_of_last_alert[app_user_list[i]]>=0
            vac_payoff += γ^(-1*(t - time_of_last_alert[app_user_list[i]]))* (η + dot(ρ_en,soc_nbrs_vac) + total_infections*ω_en) 
        end
        
        if u_vac[i]
            if rand(RNG) < 1 - Φ(vac_payoff,β)
                u_next_vac[i] = !u_vac[i]
            else
                u_next_vac[i] = u_vac[i]
            end
        else
            if rand(RNG) < Φ(vac_payoff,β)
                u_next_vac[i] = !u_vac[i]
            else
                u_next_vac[i] = u_vac[i]
            end
        end
    end
end


function agents_step!(t,modelsol)

    remake!(modelsol.inf_network,modelsol.index_vectors,modelsol.ws_matrix_tuple.daily)
    remake!(modelsol.soc_network,modelsol.index_vectors,modelsol.rest_matrix_tuple.daily)
    for network in modelsol.inf_network.graph_list[t]
        sample_mixing_graph!(network) #get new contact weights
    end

    update_alert_durations!(t,modelsol)
    update_vaccination_opinion_state!(t,modelsol,modelsol.status_totals[Int(Infected)])
    update_infection_state!(t,modelsol)

    modelsol.u_vac .= modelsol.u_next_vac
    modelsol.u_inf .= modelsol.u_next_inf
end



function solve!(modelsol,recording)
    for t in 1:modelsol.sim_length
        #advance agent states based on the new network
        record!(t,modelsol,recording)
        agents_step!(t,modelsol) 
    end
end


function Φ(payoff,β)
    return 1 / (exp(-1*β*payoff))
end

